@@ -9,17 +9,51 @@ import shutil
 import subprocess
 import apt_pkg
 import unittest
-from DistUpgrade.DistUpgradeController import DistUpgradeController
+from DistUpgrade.DistUpgradeController import (
+    DistUpgradeController,
+    component_ordering_key,
+)
 from DistUpgrade.DistUpgradeViewNonInteractive import DistUpgradeViewNonInteractive
 from DistUpgrade import DistUpgradeConfigParser
 from DistUpgrade.utils import url_downloadable
 import logging
+import mock
 
 DistUpgradeConfigParser.CONFIG_OVERRIDE_DIR = None
 
 CURDIR = os.path.dirname(os.path.abspath(__file__))
 
 
+class TestComponentOrdering(unittest.TestCase):
+
+    def test_component_ordering_key_from_set(self):
+        self.assertEqual(
+            sorted(set(["x", "restricted", "main"]),
+                   key=component_ordering_key),
+            ["main", "restricted", "x"])
+
+    def test_component_ordering_key_from_list(self):
+        self.assertEqual(
+            sorted(["x", "main"], key=component_ordering_key),
+            ["main", "x"])
+        self.assertEqual(
+            sorted(["restricted", "main"],
+                   key=component_ordering_key),
+            ["main", "restricted"])
+        self.assertEqual(
+            sorted(["main", "restricted"],
+                   key=component_ordering_key),
+            ["main", "restricted"])
+        self.assertEqual(
+            sorted(["main", "multiverse", "restricted", "universe"],
+                   key=component_ordering_key),
+            ["main", "restricted", "universe", "multiverse"])
+        self.assertEqual(
+            sorted(["a", "main", "multiverse", "restricted", "universe"],
+                   key=component_ordering_key),
+            ["main", "restricted", "universe", "multiverse", "a"])
+
+
 class TestSourcesListUpdate(unittest.TestCase):
 
     testdir = os.path.abspath(CURDIR + "/data-sources-list-test/")
@@ -79,7 +113,7 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
             ["cmp",
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".in",
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".distUpgrade"
-            ]))
+             ]))
 
     def test_commercial_transition(self):
         """
@@ -100,9 +134,9 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
         self._verifySources("""
 deb http://archive.canonical.com/ubuntu gutsy partner
 """)
-        
+
     def test_powerpc_transition(self):
-        """ 
+        """
         test transition of powerpc to ports.ubuntu.com
         """
         arch = apt_pkg.config.find("APT::Architecture")
@@ -126,7 +160,7 @@ deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security main restricted univers
         apt_pkg.config.set("APT::Architecture", arch)
 
     def test_sparc_transition(self):
-        """ 
+        """
         test transition of sparc to ports.ubuntu.com
         """
         arch = apt_pkg.config.find("APT::Architecture")
@@ -154,22 +188,22 @@ deb http://ports.ubuntu.com/ubuntu-ports/ hardy-security main restricted univers
     def testVerifySourcesListEntry(self):
         from aptsources.sourceslist import SourceEntry
         v = DistUpgradeViewNonInteractive()
-        d = DistUpgradeController(v,datadir=self.testdir)
+        d = DistUpgradeController(v, datadir=self.testdir)
         for scheme in ["http"]:
-            entry = "deb %s://archive.ubuntu.com/ubuntu/ hardy main universe restricted multiverse" % scheme
+            entry = "deb %s://archive.ubuntu.com/ubuntu/ precise main universe restricted multiverse" % scheme
             self.assertTrue(d._sourcesListEntryDownloadable(SourceEntry(entry)),
                             "entry '%s' not downloadable" % entry)
             entry = "deb %s://archive.ubuntu.com/ubuntu/ warty main universe restricted multiverse" % scheme
             self.assertFalse(d._sourcesListEntryDownloadable(SourceEntry(entry)),
-                            "entry '%s' not downloadable" % entry)
+                             "entry '%s' not downloadable" % entry)
             entry = "deb %s://archive.ubuntu.com/ubuntu/ xxx main" % scheme
             self.assertFalse(d._sourcesListEntryDownloadable(SourceEntry(entry)),
-                            "entry '%s' not downloadable" % entry)
+                             "entry '%s' not downloadable" % entry)
 
     def testEOL2EOLUpgrades(self):
         " test upgrade from EOL release to EOL release "
         v = DistUpgradeViewNonInteractive()
-        d = DistUpgradeController(v,datadir=self.testdir)
+        d = DistUpgradeController(v, datadir=self.testdir)
         shutil.copy(os.path.join(self.testdir, "sources.list.EOL"),
                     os.path.join(self.testdir, "sources.list"))
         apt_pkg.config.set("Dir::Etc::sourceparts",
@@ -206,17 +240,17 @@ deb http://old-releases.ubuntu.com/ubuntu hoary-security main restricted univers
                            os.path.join(self.testdir, "sources.list.d"))
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
-        d.fromDist = "gutsy"
-        d.toDist = "hardy"
+        d.fromDist = "oneiric"
+        d.toDist = "precise"
         d.openCache(lock=False)
         res = d.updateSourcesList()
         self.assertTrue(res)
         self._verifySources("""
 # main repo
-deb http://us.archive.ubuntu.com/ubuntu hardy main restricted multiverse universe
-deb-src http://us.archive.ubuntu.com/ubuntu hardy main restricted multiverse
+deb http://us.archive.ubuntu.com/ubuntu precise main restricted multiverse universe
+deb-src http://us.archive.ubuntu.com/ubuntu precise main restricted multiverse
 
-deb http://us.archive.ubuntu.com/ubuntu hardy-security main restricted universe multiverse
+deb http://us.archive.ubuntu.com/ubuntu precise-security main restricted universe multiverse
 """)
 
     def testEOL2SupportedUpgrade(self):
@@ -230,17 +264,17 @@ deb http://us.archive.ubuntu.com/ubuntu hardy-security main restricted universe
                            os.path.join(self.testdir, "sources.list.d"))
         v = DistUpgradeViewNonInteractive()
         d = DistUpgradeController(v, datadir=self.testdir)
-        d.fromDist = "gutsy"
-        d.toDist = "hardy"
+        d.fromDist = "oneiric"
+        d.toDist = "precise"
         d.openCache(lock=False)
         res = d.updateSourcesList()
         self.assertTrue(res)
         self._verifySources("""
 # main repo
-deb http://archive.ubuntu.com/ubuntu hardy main restricted multiverse universe
-deb-src http://archive.ubuntu.com/ubuntu hardy main restricted multiverse
+deb http://archive.ubuntu.com/ubuntu precise main restricted multiverse universe
+deb-src http://archive.ubuntu.com/ubuntu precise main restricted multiverse
 
-deb http://archive.ubuntu.com/ubuntu hardy-security main restricted universe multiverse
+deb http://archive.ubuntu.com/ubuntu precise-security main restricted universe multiverse
 """)
 
     def test_partner_update(self):
@@ -346,6 +380,51 @@ deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted
 deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 # A PPA with a unicode comment
 # deb http://ppa.launchpad.net/random-ppa quantal main # ppa of Víctor R. Ruiz (vrruiz) disabled on upgrade to gutsy
+""")
+
+    def test_local_mirror(self):
+        """
+        test that a local mirror with official -backports works (LP:# 1067393)
+        """
+        shutil.copy(os.path.join(self.testdir, "sources.list.local"),
+                    os.path.join(self.testdir, "sources.list"))
+        apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
+        v = DistUpgradeViewNonInteractive()
+        d = DistUpgradeController(v, datadir=self.testdir)
+        d.openCache(lock=False)
+        res = d.updateSourcesList()
+        self.assertTrue(res)
+
+        # verify it
+        self._verifySources("""
+deb http://192.168.1.1/ubuntu gutsy main restricted
+deb http://192.168.1.1/ubuntu gutsy-updates main restricted
+deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
+deb http://archive.ubuntu.com/ubuntu gutsy-backports main restricted universe multiverse
+""")
+
+    def test_disable_proposed(self):
+        """
+        Test that proposed is disabled when upgrading to a development
+        release.
+        """
+        shutil.copy(os.path.join(self.testdir,
+                    "sources.list.proposed_enabled"),
+                    os.path.join(self.testdir, "sources.list"))
+        apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
+        v = DistUpgradeViewNonInteractive()
+        options = mock.Mock()
+        options.devel_release = True
+        d = DistUpgradeController(v, options, datadir=self.testdir)
+        d.openCache(lock=False)
+        res = d.updateSourcesList()
+        self.assertTrue(res)
+
+        self._verifySources("""
+deb http://archive.ubuntu.com/ubuntu gutsy main restricted
+deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted
+deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
+# deb http://archive.ubuntu.com/ubuntu gutsy-proposed universe main multiverse restricted #Not for humans during development stage of release gutsy
 """)
 
     def _verifySources(self, expected):
@@ -355,7 +434,8 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
             self.assertTrue(
                 l in sources_list.split("\n"),
                 "expected entry '%s' in sources.list missing. got:\n'''%s'''" %
-                    (l, sources_list))
+                (l, sources_list))
+
 
 if __name__ == "__main__":
     import sys
